@@ -152,10 +152,41 @@ HtmlParser::decode_entities(string &s)
 void
 HtmlParser::parse(const string &body)
 {
+    // Check for BOM.
+    string::const_iterator begin_after_bom = body.begin();
+    if (body.size() >= 3) {
+	switch (body[0]) {
+	  case '\xef':
+	    if (body[1] == '\xbb' && body[2] == '\xbf') {
+		charset = "utf-8";
+		begin_after_bom += 3;
+	    }
+	    break;
+	  case '\xfe':
+	    if (body[1] == '\xff') {
+		string utf8_body(body, 2);
+		convert_to_utf8(utf8_body, "utf-16le");
+		charset = "utf-8";
+		parse(utf8_body);
+		return;
+	    }
+	    break;
+	  case '\xff':
+	    if (body[1] == '\xfe') {
+		string utf8_body(body, 2);
+		convert_to_utf8(utf8_body, "utf-16be");
+		charset = "utf-8";
+		parse(utf8_body);
+		return;
+	    }
+	    break;
+	}
+    }
+
     in_script = false;
 
     parameters.clear();
-    string::const_iterator start = body.begin();
+    string::const_iterator start = begin_after_bom;
 
     while (true) {
 	// Skip through until we find an HTML tag, a comment, or the end of
@@ -173,8 +204,7 @@ HtmlParser::parse(const string &body)
 	    if (ch == '?') {
 		// PHP code or XML declaration.
 		// XML declaration is only valid at the start of the first line.
-		// FIXME: need to deal with BOMs...
-		if (p != body.begin() || body.size() < 20) break;
+		if (p != begin_after_bom || body.size() < 20) break;
 
 		// XML declaration looks something like this:
 		// <?xml version="1.0" encoding="UTF-8"?>
