@@ -21,6 +21,7 @@
 #include <config.h>
 
 #include <cstdlib>
+#include <cstring>
 #include <iostream>
 #include <string>
 
@@ -36,6 +37,10 @@ struct testcase {
     const char * sample;
 };
 
+// Wide character test data is signalled by a single leading nul and terminated
+// by a double nul.
+#define WIDE(X) "\0" X "\0"
+
 static const testcase tests[] = {
     { "<body>test<!--htdig_noindex-->icle<!--/htdig_noindex-->s</body>",
       "tests", "", "", "" },
@@ -52,6 +57,11 @@ static const testcase tests[] = {
     { "<html><head><meta http-equiv=Content-Type content=\"text/html;charset=utf-8\"><title>\xc2\xae</title></head><body>\xc2\xa3</body></html>", "\xc2\xa3", "\xc2\xae", "", "" },
     { "<html><head><meta charset='utf-8'><title>\xc2\xae</title></head><body>\xc2\xa3</body></html>", "\xc2\xa3", "\xc2\xae", "", "" },
     { "<html><head><title>\xc2\xae</title><meta charset=\"utf-8\"></head><body>\xc2\xa3</body></html>", "\xc2\xa3", "\xc2\xae", "", "" },
+    // The UTF-8 "BOM" should also set the charset to utf-8.
+    { "\xef\xbb\xbf<html><head><title>\xc2\xae</title></head><body>\xc2\xa3</body></html>", "\xc2\xa3", "\xc2\xae", "", "" },
+    { "<title>X</title>", "", "X", "", "" },
+    { WIDE("\xfe\xff<\0t\0i\0t\0l\0e\0>\0\x20\x26<\0/\0t\0i\0t\0l\0e\0>\0"), "", "\xe2\x98\xa0", "", "" },
+    { WIDE("\xff\xfe\0<\0t\0i\0t\0l\0e\0>\x26\x20\0<\0/\0t\0i\0t\0l\0e\0>"), "", "\xe2\x98\xa0", "", "" },
     { "<html><body><p>This is \nthe text</p><p>This is \nthe tex</p></body></html>", "This is the text\rThis is the tex", "", "", "" },
     // Check we default to UTF-8 for HTML5.
     { "<!DOCTYPE html><html><head><title>\xc2\xae</title></head><body>\xc2\xa3</body></html>", "\xc2\xa3", "\xc2\xae", "", "" },
@@ -85,11 +95,22 @@ main()
 {
     for (size_t i = 0; tests[i].html; ++i) {
 	MyHtmlParser p;
+	const char* html_begin = tests[i].html;
+	size_t html_len = strlen(html_begin);
+	if (html_len == 0) {
+	    // Wide character test data is signalled by a single leading nul
+	    // and terminated by a double nul.
+	    ++html_begin;
+	    while (html_begin[html_len] || html_begin[html_len + 1]) {
+		html_len += 2;
+	    }
+	}
+	string html(html_begin, html_len);
 	try {
-	    p.parse_html(tests[i].html, "iso-8859-1", false);
+	    p.parse_html(html, "iso-8859-1", false);
 	} catch (const string &newcharset) {
 	    p.reset();
-	    p.parse_html(tests[i].html, newcharset, true);
+	    p.parse_html(html, newcharset, true);
 	}
 	if (!p.indexing_allowed) {
 	    cout << "indexing disallowed by meta tag - skipping\n";
